/* This is ColorControl.c
	 1999, 2000, 2001 By Kyle Hammond
	CodeBlazer Technologies, Ltd	*/
#include "ColorControl.h"

#define kColorControl_ColorOffset		0
#define kColorControl_StateOffset		4

static LRESULT CALLBACK local_windowProc( IN HWND inHWnd, IN UINT inMsg, IN WPARAM inWParam, IN LPARAM inLParam );
static void local_redrawWindow( IN HWND inHWnd );
static void local_uncheckAllOthersInGroup( IN HWND inHWnd );
static BOOL local_chooseColor( IN HWND inOwningWnd, COLORREF *ioCustomColors, COLORREF *ioColor );

static BOOL _gColorControl_Registered = false;
static COLORREF _gColorControl_CustomColors[ 16 ];

DWORD InitColorControl( HINSTANCE inOwningInstance )
{	WNDCLASSEX	windowClass; /* Window class */
	DWORD		err = NO_ERROR;

	if ( !_gColorControl_Registered ) {
		/* If we're not registered yet, do so now. */
		windowClass.cbSize = sizeof( windowClass );

		windowClass.style = NULL; /* No special style */
		windowClass.lpfnWndProc = local_windowProc;
		windowClass.cbClsExtra = 0; /* No extra class data. */
		windowClass.cbWndExtra = 8;		/* Extra window data is: COLORREF and selection state; so we have two longs */
		windowClass.hInstance = inOwningInstance; /* Current instance is the owner */

		windowClass.hIcon = NULL; /* Set icon */
		windowClass.hCursor = LoadCursor( NULL, IDC_ARROW ); /* Set cursor to stock arrow cursor */
		windowClass.hbrBackground = NULL; /* Set window background brush to none - we'll draw our own color */

		windowClass.lpszMenuName = NULL; /* No menu */
		windowClass.lpszClassName = WC_COLORCONTROL;

		windowClass.hIconSm = NULL;

		if ( RegisterClassEx( &windowClass ) != 0 ) /* Register the class */
			err = GetLastError( );
		else
			_gColorControl_Registered = true;
	}

	return err;
}

static LRESULT CALLBACK local_windowProc( IN HWND inHWnd, IN UINT inMsg, IN WPARAM inWParam, IN LPARAM inLParam )
{	LRESULT		result = 0;
	LONG			tempLong = RGB( 0, 0, 0 );
	LONG_PTR		tempLongPtr;

	switch ( inMsg ) {
	case WM_CREATE:
		/* Initialize the window to have a black color and not be selected. */
		SetWindowLongPtr( inHWnd, kColorControl_ColorOffset, tempLong );
		SetWindowLongPtr( inHWnd, kColorControl_StateOffset, 0 );
		break;

	case WM_PAINT:
		local_redrawWindow( inHWnd );
		break;

	case WM_LBUTTONDOWN:
		/* Should track the mouse and show the selection on/off stuff. */
		tempLongPtr = GetWindowLongPtr( inHWnd, GWL_STYLE );

		if ( ( tempLongPtr & KCCS_CHOOSEONSINGLE ) != 0 ) {
			tempLong = GetWindowLongPtr( inHWnd, kColorControl_ColorOffset );
			if ( local_chooseColor( inHWnd, _gColorControl_CustomColors, (COLORREF *)&tempLong ) )
				SendMessage( inHWnd, KCCM_SETCOLOR, tempLong, 0 );
		} else if ( ( tempLongPtr & KCCS_AUTORADIOSELECT ) != 0 ) {
			/* NOTE: We cannot disable the button here since at least one in the group has to be on. */
			tempLong = GetWindowLongPtr( inHWnd, kColorControl_StateOffset );
			if ( !tempLong ) {
				/* Turn on the control. */
				SetWindowLongPtr( inHWnd, kColorControl_StateOffset, true );
				RedrawWindow( inHWnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN );

				/* And turn off all others in our group. */
				local_uncheckAllOthersInGroup( inHWnd );
			}
		} else if ( ( tempLongPtr & KCCS_AUTOSELECT ) != 0 ) {
			/* Change the state of the control. */
			SetWindowLongPtr( inHWnd, kColorControl_StateOffset, !GetWindowLongPtr( inHWnd, kColorControl_StateOffset ) );
			RedrawWindow( inHWnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN );
		} else if ( ( tempLongPtr & KCCS_SELECT ) != 0 )
			/* Send a notification to the parent window. */
			SendMessage( (HWND)GetWindowLongPtr( inHWnd, GWLP_HWNDPARENT ), WM_COMMAND, MAKEWPARAM( BN_CLICKED, 0 ), (LPARAM)inHWnd );
		break;

	case WM_LBUTTONDBLCLK:
		tempLongPtr = GetWindowLongPtr( inHWnd, GWL_STYLE );

		if ( ( tempLongPtr & KCCS_CHOOSEONDOUBLE ) != 0 ) {
			tempLong = GetWindowLongPtr( inHWnd, kColorControl_ColorOffset );
			if ( local_chooseColor( inHWnd, _gColorControl_CustomColors, (COLORREF *)&tempLong ) )
				SendMessage( inHWnd, KCCM_SETCOLOR, tempLong, 0 );
		}
		break;

	case KCCM_SETCOLOR:
		tempLong = (COLORREF)inWParam;
		SetWindowLongPtr( inHWnd, kColorControl_ColorOffset, tempLong );
		RedrawWindow( inHWnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN );
/*	Notify parent window that the color has changed.
		SendMessage( (HWND)GetWindowLongPtr( inHWnd, GWLP_HWNDPARENT ), WM_COMMAND, MAKEWPARAM( BN_CLICKED, 0 ), (LPARAM)inHWnd ); */
		break;
	case KCCM_GETCOLOR:
		tempLongPtr = GetWindowLongPtr( inHWnd, kColorControl_ColorOffset );
		result = (COLORREF)tempLongPtr;
		break;

	case KCCM_GETCHECK:
		tempLongPtr = GetWindowLongPtr( inHWnd, kColorControl_StateOffset );
		if ( tempLongPtr != NULL )
			result = BST_CHECKED;
		else
			result = BST_UNCHECKED;
		break;
	case KCCM_SETCHECK:
		SetWindowLongPtr( inHWnd, kColorControl_StateOffset, inWParam );
		RedrawWindow( inHWnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN );
		break;

	default:
		result = DefWindowProc( inHWnd, inMsg, inWParam, inLParam );
		break;
	}

	return result;
}

static void local_redrawWindow( IN HWND inHWnd )
{	PAINTSTRUCT	paintInfo;
	RECT			box;
	HBRUSH		brush = NULL;
	LONG_PTR		tempLongPtr, windowStyles;

	if ( BeginPaint( inHWnd, &paintInfo ) != 0 ) {
		GetClientRect( inHWnd, &box );

		/* Get the color; create a solid brush and fill with that color. */
		tempLongPtr = GetWindowLongPtr( inHWnd, kColorControl_ColorOffset );
		if ( tempLongPtr != NULL )
			brush = CreateSolidBrush( (COLORREF)tempLongPtr );

		if ( brush != NULL ) {
			FillRect( paintInfo.hdc, &box, brush );

			DeleteObject( brush );
		} else
			FillRect( paintInfo.hdc, &box, (HBRUSH)GetStockObject( BLACK_BRUSH ) );

		/* Get the styles of the window */
		windowStyles = GetWindowLongPtr( inHWnd, GWL_STYLE );

		if ( ( windowStyles & KCCS_SELECT ) != 0 || ( windowStyles & KCCS_AUTOSELECT ) != 0 || ( windowStyles & KCCS_AUTORADIOSELECT ) != 0 ) {
			/* Now, draw a selection if the window is selected. */
			tempLongPtr = GetWindowLongPtr( inHWnd, kColorControl_StateOffset );
			if ( tempLongPtr != NULL ) {
				FrameRect( paintInfo.hdc, &box, (HBRUSH)GetStockObject( WHITE_BRUSH ) );
				InflateRect( &box, -1, -1 );
				FrameRect( paintInfo.hdc, &box, (HBRUSH)GetStockObject( BLACK_BRUSH ) );
				/* Inflate back to the client rect if needed.
				InflateRect( &box, 1, 1 );*/
			}
		}

		EndPaint( inHWnd, &paintInfo );
	}
}

static void local_uncheckAllOthersInGroup( IN HWND inHWnd )
{	HWND	parentWnd, tempWnd;

	parentWnd = (HWND)GetWindowLongPtr( inHWnd, GWLP_HWNDPARENT );

	/* Go forward through the group, disabling as we go. */
	tempWnd = inHWnd;
	while ( ( tempWnd = GetNextDlgGroupItem( parentWnd, tempWnd, true ) ) != inHWnd )
		SendMessage( tempWnd, KCCM_SETCHECK, 0, 0 );
}

/* Note: ioCustomColors should be an array of 16 COLORREFs. */
static BOOL local_chooseColor( IN HWND inOwningWnd, COLORREF *ioCustomColors, COLORREF *ioColor )
{	CHOOSECOLOR	choice;
	bool			result = false;
	ULONG		err = NO_ERROR;

	if ( ioColor == NULL )
		return false;

	choice.lStructSize = sizeof( choice );
	choice.hwndOwner = inOwningWnd;
	choice.hInstance = NULL;
	choice.rgbResult = *ioColor;
	choice.lpCustColors = ioCustomColors;
	choice.Flags = CC_RGBINIT;
	choice.lCustData = NULL;
	choice.lpfnHook = NULL;
	choice.lpTemplateName = NULL;

	result = ( ChooseColor( &choice ) != 0 );

	if ( !result ) {
		err = CommDlgExtendedError( );
		SetLastError( err );
	} else
		*ioColor = choice.rgbResult;

	return result;
}
